/*****************************************************************************

  FileName:     5ESSStateNT.c

  Contents:     AT&T 5ESS ISDN State Engine for NT (Network Mode).

	            The controlling state engine for Q.931 is the state engine
	            on the NT side. The state engine on the TE side is a slave 
	            of this. The TE side maintain it's own states as described in
	            ITU-T Q931, but will in    raise conditions be overridden by 
	            the NT side.

  License/Copyright:

  Copyright (c) 2007, Jan Vidar Berger, Case Labs, Ltd. All rights reserved.
  email:janvb@caselaboratories.com  

  Copyright (c) 2007, Michael Jerris. All rights reserved.
  email:mike@jerris.com  
  
  Copyright (c) 2007, Michael S. Collins, All rights reserved.
  email:mcollins@fcnetwork.com
  
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are 
  met:

	* Redistributions of source code must retain the above copyright notice, 
	  this list of conditions and the following disclaimer.
	* Redistributions in binary form must reproduce the above copyright notice, 
	  this list of conditions and the following disclaimer in the documentation 
	  and/or other materials provided with the distribution.
	* Neither the name of the Case Labs, Ltd nor the names of its contributors 
	  may be used to endorse or promote products derived from this software 
	  without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

#define Q931PRIVATE

#include "5ESS.h"

extern L3INT Q931L4HeaderSpace;
extern struct Q931MessageIE *ATT5ESSMessageIEs;

/*
 * ProtDisc 3 (Service/ServiceAck) and 8 (Connect/ConnectAck) share the same message IDs
 * Use custom functions to handle them
 */
static L3INT ATT5ESSProc0x07NT(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom);
static L3INT ATT5ESSProc0x0fNT(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom);


void ATT5ESSCreateNT(struct Q931Dialect *dlct)
{
	Q931DialectSetName(dlct, "5ESS NT");

	/*
	 * Flags
	 */
	Q931DialectSetFlag(dlct, 0);

	/*
	 * Codesets
	 */
	Q931DialectAddCodeset(dlct, Q931_CODESET_0);
	Q931DialectAddCodeset(dlct, Q931_CODESET_6);
	Q931DialectAddCodeset(dlct, Q931_CODESET_7);

	/*
	 *
	 */
	Q931DialectSetMesProc(dlct, Q931mes_ALERTING,             Q931ProcAlertingNT,          Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CALL_PROCEEDING,      Q931ProcCallProceedingNT,    Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CONNECT,              ATT5ESSProc0x07NT,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CONNECT_ACKNOWLEDGE,  ATT5ESSProc0x0fNT,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_PROGRESS,             Q931ProcProgressNT,          Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SETUP,                Q931ProcSetupNT,             Q931Umes_Generic, Q931Pmes_Generic);
#if 0
	Q931DialectSetMesProc(dlct, Q931mes_SETUP_ACKNOWLEDGE,    Q931ProcSetupAckNT,          Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESUME,               Q931ProcResumeNT,            Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESUME_ACKNOWLEDGE,   Q931ProcResumeAckNT,         Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESUME_REJECT,        Q931ProcResumeRejectNT,      Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SUSPEND,              Q931ProcSuspendNT,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SUSPEND_ACKNOWLEDGE,  Q931ProcSuspendAckNT,        Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SUSPEND_REJECT,       Q931ProcSuspendRejectNT,     Q931Umes_Generic, Q931Pmes_Generic);
#endif
	Q931DialectSetMesProc(dlct, Q931mes_USER_INFORMATION,     Q931ProcUserInformationNT,   Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_DISCONNECT,           Q931ProcDisconnectNT,        Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RELEASE,              Q931ProcReleaseNT,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RELEASE_COMPLETE,     Q931ProcReleaseCompleteNT,   Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESTART,              Q931ProcRestartNT,           Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_RESTART_ACKNOWLEDGE,  Q931ProcRestartAckNT,        Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_CONGESTION_CONTROL,   Q931ProcCongestionControlNT, Q931Umes_Generic, Q931Pmes_Generic);
#if 0
	Q931DialectSetMesProc(dlct, Q931mes_INFORMATION,          Q931ProcInformationNT,       Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_NOTIFY,               Q931ProcNotifyNT,            Q931Umes_Generic, Q931Pmes_Generic);
#endif
	Q931DialectSetMesProc(dlct, Q931mes_STATUS,               Q931ProcStatusNT,            Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_STATUS_ENQUIRY,       Q931ProcStatusEnquiryNT,     Q931Umes_Generic, Q931Pmes_Generic);
	Q931DialectSetMesProc(dlct, Q931mes_SEGMENT,              Q931ProcSegmentNT,           Q931Umes_Generic, Q931Pmes_Generic);

	Q931DialectSetMesProc(dlct, Q932mes_FACILITY,             Q932ProcFacilityNT,          Q932Umes_Facility,       Q932Pmes_Facility);
	Q931DialectSetMesProc(dlct, Q932mes_HOLD,                 Q932ProcHoldNT,              Q932Umes_Hold,           Q932Pmes_Hold);
	Q931DialectSetMesProc(dlct, Q932mes_HOLD_ACKNOWLEDGE,     Q932ProcHoldAckNT,           Q932Umes_HoldAck,        Q932Pmes_HoldAck);
	Q931DialectSetMesProc(dlct, Q932mes_HOLD_REJECT,          Q932ProcHoldRejectNT,        Q932Umes_HoldReject,     Q932Pmes_HoldReject);
	Q931DialectSetMesProc(dlct, Q932mes_REGISTER,             Q932ProcRegisterNT,          Q932Umes_Register,       Q932Pmes_Register);
	Q931DialectSetMesProc(dlct, Q932mes_RETRIEVE,             Q932ProcRetrieveNT,          Q932Umes_Retrieve,       Q932Pmes_Retrieve);
	Q931DialectSetMesProc(dlct, Q932mes_RETRIEVE_ACKNOWLEDGE, Q932ProcRetrieveAckNT,       Q932Umes_RetrieveAck,    Q932Pmes_RetrieveAck);
	Q931DialectSetMesProc(dlct, Q932mes_RETRIEVE_REJECT,      Q932ProcRetrieveRejectNT,    Q932Umes_RetrieveReject, Q932Pmes_RetrieveReject);

	/*
	 * Unknown / Invalid (Unexpected) Message handler
	 */
	Q931DialectSetUnknownMesProc(dlct, Q931ProcUnknownMessage);
	Q931DialectSetInvalidMesProc(dlct, Q931ProcUnexpectedMessage);


	/*
	 * Set up the IE encoder/decoder handle table.
	 */ 
	Q931DialectSetIEProc(dlct, Q931ie_SEGMENTED_MESSAGE,                Q931Pie_Segment,       Q931Uie_Segment);
	Q931DialectSetIEProc(dlct, Q931ie_BEARER_CAPABILITY,                Q931Pie_BearerCap,     Q931Uie_BearerCap);
	Q931DialectSetIEProc(dlct, Q931ie_CAUSE,                            Q931Pie_Cause,         Q931Uie_Cause);
	Q931DialectSetIEProc(dlct, Q931ie_CALL_IDENTITY,                    Q931Pie_CallID,        Q931Uie_CallID);
	Q931DialectSetIEProc(dlct, Q931ie_CALL_STATE,                       Q931Pie_CallState,     Q931Uie_CallState);
	Q931DialectSetIEProc(dlct, Q931ie_CHANGE_STATUS,                    Q931Pie_ChangeStatus,  Q931Uie_ChangeStatus);
	Q931DialectSetIEProc(dlct, Q931ie_CHANNEL_IDENTIFICATION,           Q931Pie_ChanID,        Q931Uie_ChanID);
	Q931DialectSetIEProc(dlct, Q931ie_PROGRESS_INDICATOR,               Q931Pie_ProgInd,       Q931Uie_ProgInd);
	Q931DialectSetIEProc(dlct, Q931ie_NETWORK_SPECIFIC_FACILITIES,      Q931Pie_NetFac,        Q931Uie_NetFac);
	Q931DialectSetIEProc(dlct, Q931ie_NOTIFICATION_INDICATOR,           Q931Pie_NotifInd,      Q931Uie_NotifInd);
	Q931DialectSetIEProc(dlct, Q931ie_DISPLAY,                          Q931Pie_Display,       Q931Uie_Display);
	Q931DialectSetIEProc(dlct, Q931ie_DATETIME,                         Q931Pie_DateTime,      Q931Uie_DateTime);
	Q931DialectSetIEProc(dlct, Q931ie_KEYPAD_FACILITY,                  Q931Pie_KeypadFac,     Q931Uie_KeypadFac);
	Q931DialectSetIEProc(dlct, Q931ie_SIGNAL,                           Q931Pie_Signal,        Q931Uie_Signal);
	Q931DialectSetIEProc(dlct, Q931ie_TRANSIT_DELAY_SELECTION_AND_IND,  Q931Pie_TransNetSel,   Q931Uie_TransNetSel);
	Q931DialectSetIEProc(dlct, Q931ie_CALLING_PARTY_NUMBER,             Q931Pie_CallingNum,    Q931Uie_CallingNum);
	Q931DialectSetIEProc(dlct, Q931ie_CALLING_PARTY_SUBADDRESS,         Q931Pie_CallingSub,    Q931Uie_CallingSub);
	Q931DialectSetIEProc(dlct, Q931ie_CALLED_PARTY_NUMBER,              Q931Pie_CalledNum,     Q931Uie_CalledNum);
	Q931DialectSetIEProc(dlct, Q931ie_CALLED_PARTY_SUBADDRESS,          Q931Pie_CalledSub,     Q931Uie_CalledSub);
	Q931DialectSetIEProc(dlct, Q931ie_TRANSIT_NETWORK_SELECTION,        Q931Pie_TransNetSel,   Q931Uie_TransNetSel);
	Q931DialectSetIEProc(dlct, Q931ie_RESTART_INDICATOR,                Q931Pie_RestartInd,    Q931Uie_RestartInd);
	Q931DialectSetIEProc(dlct, Q931ie_LOW_LAYER_COMPATIBILITY,          Q931Pie_LLComp,        Q931Uie_LLComp);
	Q931DialectSetIEProc(dlct, Q931ie_HIGH_LAYER_COMPATIBILITY,         Q931Pie_HLComp,        Q931Uie_HLComp);
	Q931DialectSetIEProc(dlct, Q931ie_USER_USER,                        Q931Pie_UserUser,      Q931Uie_UserUser);
	Q931DialectSetIEProc(dlct, Q931ie_GENERIC_DIGITS,                   Q931Pie_GenericDigits, Q931Uie_GenericDigits);

	Q931DialectSetIEProc(dlct, Q931ie_CONNECTED_NUMBER,                 Q931Pie_Generic,       Q931Uie_Generic);
	Q931DialectSetIEProc(dlct, Q931ie_FACILITY,                         Q931Pie_Generic,       Q931Uie_Generic);


	/*
	 * State Table
	 */

	/* Any State */
	Q931DialectAddStateEntry(dlct, Q931_NANY, Q931mes_STATUS,           MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_NANY, Q931mes_STATUS_ENQUIRY,   MSGF_FROM_L2);

	/* State 0 Idle */
	Q931DialectAddStateEntry(dlct, Q931_N0, Q931mes_SETUP,            MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N0, Q931mes_RELEASE,          MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N0, Q931mes_RELEASE_COMPLETE, MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N0, Q931mes_RESUME,           MSGF_FROM_L2);

	/* State 1 Call Initiating */
	Q931DialectAddStateEntry(dlct, Q931_N1, Q931mes_DISCONNECT,        MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N1, Q931mes_SETUP_ACKNOWLEDGE, MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N1, Q931mes_RELEASE_COMPLETE,  MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N1, Q931mes_CALL_PROCEEDING,   MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N1, Q931mes_ALERTING,          MSGF_FROM_L4); /* ITU-T Q.931 Annex D */
	Q931DialectAddStateEntry(dlct, Q931_N1, Q931mes_CONNECT,           MSGF_FROM_L4); /* ITU-T Q.931 Annex D */

	/* State 2 Overlap Sending */
	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_DISCONNECT,      MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_INFORMATION,     MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_CALL_PROCEEDING, MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_ALERTING,        MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_PROGRESS,        MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_CONNECT,         MSGF_FROM_L4);
/*	Q931DialectAddStateEntry(dlct, Q931_N2, Q931mes_RELEASE,         MSGF_FROM_L4); ??? */

	/* State 3 Outgoing Call Proceeding */
	Q931DialectAddStateEntry(dlct, Q931_N3, Q931mes_DISCONNECT,     MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N3, Q931mes_PROGRESS,       MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N3, Q931mes_ALERTING,       MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N3, Q931mes_CONNECT,        MSGF_FROM_L4);
/*	Q931DialectAddStateEntry(dlct, Q931_N3, Q931mes_RELEASE,        MSGF_FROM_L4); ??? */

	/* State 4 Call Delivered */
	Q931DialectAddStateEntry(dlct, Q931_N4, Q931mes_DISCONNECT,     MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N4, Q931mes_CONNECT,        MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N4, Q931mes_PROGRESS,       MSGF_FROM_L4);

	/* State 6 Call Present */
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_CONNECT,           MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_RELEASE_COMPLETE,  MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_SETUP_ACKNOWLEDGE, MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_INFORMATION,       MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_ALERTING,          MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_CALL_PROCEEDING,   MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_RELEASE,           MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N6, Q931mes_DISCONNECT,        MSGF_FROM_L2);

	/* State 7 Call Received */
	Q931DialectAddStateEntry(dlct, Q931_N7, Q931mes_CONNECT,        MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N7, Q931mes_DISCONNECT,     MSGF_FROM_BOTH);

	/* State 8 Connect request */
	Q931DialectAddStateEntry(dlct, Q931_N8, Q931mes_CONNECT_ACKNOWLEDGE, MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N8, Q931mes_DISCONNECT,          MSGF_FROM_BOTH);

	/* State 9 Incoming Call Proceeding */
	Q931DialectAddStateEntry(dlct, Q931_N9, Q931mes_CONNECT,        MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N9, Q931mes_ALERTING,       MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N9, Q931mes_PROGRESS,       MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N9, Q931mes_DISCONNECT,     MSGF_FROM_BOTH);

	/* State 10 Active */
	Q931DialectAddStateEntry(dlct, Q931_N10, Q931mes_DISCONNECT,          MSGF_FROM_BOTH);
	Q931DialectAddStateEntry(dlct, Q931_N10, Q931mes_CONNECT_ACKNOWLEDGE, MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N10, Q931mes_SUSPEND,             MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N10, Q931mes_NOTIFY,              MSGF_FROM_BOTH);

	/* State 11 Disconnect Request */
	Q931DialectAddStateEntry(dlct, Q931_N11, Q931mes_RELEASE,        MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N11, Q931mes_DISCONNECT,     MSGF_FROM_L2);
/*	Q931DialectAddStateEntry(dlct, Q931_N11, Q931mes_NOTIFY,         MSGF_FROM_L2); ??? */

	/* State 12 Disconnect Ind */
	Q931DialectAddStateEntry(dlct, Q931_N12, Q931mes_RELEASE,        MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N12, Q931mes_DISCONNECT,     MSGF_FROM_L2);

	/* State 15 Suspend Request */
	Q931DialectAddStateEntry(dlct, Q931_N15, Q931mes_SUSPEND_ACKNOWLEDGE, MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N15, Q931mes_SUSPEND_REJECT,      MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N15, Q931mes_DISCONNECT,          MSGF_FROM_BOTH);
/*	Q931DialectAddStateEntry(dlct, Q931_N15, Q931mes_RELEASE,             MSGF_FROM_L2); ??? */

	/* State 17 Resume Request */
	Q931DialectAddStateEntry(dlct, Q931_N17, Q931mes_RESUME_ACKNOWLEDGE,  MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N17, Q931mes_RESUME_REJECT,       MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N17, Q931mes_DISCONNECT,          MSGF_FROM_BOTH);

	/* State 19 Release Request */
	Q931DialectAddStateEntry(dlct, Q931_N19, Q931mes_RELEASE_COMPLETE,    MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N19, Q931mes_RELEASE,             MSGF_FROM_L2);

	/* State 25 Overlap Receiving */
/*	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_SETUP_ACKNOWLEDGE,   MSGF_FROM_L4); */
/*	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_RELEASE_COMPLETE,    MSGF_FROM_L4); */
	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_CALL_PROCEEDING,     MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_ALERTING,            MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_PROGRESS,            MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_CONNECT,             MSGF_FROM_L2);
	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_INFORMATION,         MSGF_FROM_L4);
	Q931DialectAddStateEntry(dlct, Q931_N25, Q931mes_DISCONNECT,          MSGF_FROM_BOTH);


	/*
	 * Timer default values
	 */
	Q931DialectSetTimerProcAll(dlct, Q931TimeoutDummy);

	/* Per-timer timeout callbacks, per-mode for now, until i know if they can be shared (and which) */
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T301, Q931ProcTimeoutT301TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T302, Q931ProcTimeoutT302TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T303, Q931ProcTimeoutT303TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T304, Q931ProcTimeoutT304TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T305, Q931ProcTimeoutT305TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T306, Q931ProcTimeoutT305TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T307, Q931ProcTimeoutT305TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T308, Q931ProcTimeoutT308TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T309, Q931ProcTimeoutT309TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T310, Q931ProcTimeoutT310TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T312, Q931ProcTimeoutT311TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T314, Q931ProcTimeoutT314TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T316, Q931ProcTimeoutT316TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T317, Q931ProcTimeoutT317TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T320, Q931ProcTimeoutT319TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T321, Q931ProcTimeoutT321TE);
	Q931DialectSetTimerProc(dlct, Q931_TIMER_T322, Q931ProcTimeoutT322TE);

	Q931DialectSetTimeout(dlct, Q931_TIMER_T301, 180000);	/* T301: 180s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T302,  15000);	/* T302:  15s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T303,   4000);	/* T303:   4s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T304,  20000);	/* T304:  20s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T305,  30000);	/* T305:  30s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T306,  30000);	/* T306:  30s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T307, 180000);	/* T307: 180s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T308,   4000);	/* T308:   4s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T309,  60000);	/* T309:  60s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T310,  10000);	/* T310:  10s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T312,  12000);	/* T312:  12s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T314,   4000);	/* T314:   4s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T316, 120000);	/* T316: 120s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T317,  90000);	/* T317:  90s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T320,  30000);	/* T320:  30s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T321,  30000);	/* T321:  30s */
	Q931DialectSetTimeout(dlct, Q931_TIMER_T322,   4000);	/* T322:   4s */

	/*
	 * Q.931 message <-> ie table
	 */
	Q931DialectSetMesIEMap(dlct, ATT5ESSMessageIEs);
}

/*****************************************************************************

  Function:		ATT5ESSProc0x0fNT

*****************************************************************************/
static L3INT ATT5ESSProc0x0fNT(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom)
{
	Q931mes_Header *pMes = (Q931mes_Header *)&buf[Q931L4HeaderSpace];
	L3INT ret = Q931E_NO_ERROR;

	switch (pMes->ProtDisc) {
	case 8:
		return Q931ProcConnectAckNT(pTrunk, buf, iFrom);

	case 3:
		if (iFrom == 4) {
			/* TODO Add proc here*/
			ret = Q931Tx32Data(pTrunk, 0, buf, pMes->Size);
		}
		else if (iFrom == 2) {
			/* TODO Add proc here*/
			ret = Q931Tx34(pTrunk, buf, pMes->Size);

			if (Q931TrunkIsSetFlag(pTrunk, Q931_TFLAG_AUTO_SERVICE_ACK)) {
				Q931Log(pTrunk, Q931_LOG_DEBUG, "autoServiceAck is on, responding to Service Req from network...\n");
				Q931AckService(pTrunk, buf);
			}
		}
		return ret;

	default:
		break;
	}
	return Q931E_NO_ERROR;
}

/*****************************************************************************

  Function:		ATT5ESSProc0x07TE

*****************************************************************************/
static L3INT ATT5ESSProc0x07NT(Q931_TrunkInfo_t *pTrunk, L3UCHAR * buf, L3INT iFrom)
{
	Q931mes_Header *pMes = (Q931mes_Header *)&buf[Q931L4HeaderSpace];
	L3INT ret = Q931E_NO_ERROR;

	switch (pMes->ProtDisc) {
	case 8:
		return Q931ProcConnectNT(pTrunk, buf, iFrom);

	case 3:
		if (iFrom == 4) {
			/* TODO Add proc here*/
			ret = Q931Tx32Data(pTrunk, 0, buf, pMes->Size);
		}
		else if (iFrom == 2) {
			/* TODO Add proc here*/
			ret = Q931Tx34(pTrunk, buf, pMes->Size);
		}
		return ret;

	default:
		break;
	}
	return Q931E_NO_ERROR;
}
